1// CAxCDManager.cpp: implementation of the CAxCDManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AxCDManager.h"
#include <memory>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

namespace AxWinLib {

//-------------------------------------------------------------------
DWORD CCDThread::Execute()
{
	MCI_OPEN_PARMS op;
	DWORD flags;
	TCHAR szDriveName[4];

	::memset(szDriveName , 0 , sizeof(szDriveName));
	::memset(&op, 0, sizeof(op));

	op.lpstrDeviceType = "cdaudio";
	
	::lstrcpy(szDriveName , "?:");
	szDriveName[0] = m_chDriveName;

	op.lpstrElementName = szDriveName;
	flags = MCI_OPEN_TYPE  | MCI_OPEN_ELEMENT | MCI_OPEN_SHAREABLE;

	mciSendCommand(0,MCI_OPEN,flags,(unsigned long)&op);

	if (op.wDeviceID)
	{
		if (m_bOpen)
			mciSendCommand(op.wDeviceID,MCI_SET,MCI_SET_DOOR_OPEN,0);
		else
			mciSendCommand(op.wDeviceID,MCI_SET,MCI_SET_DOOR_CLOSED,0);

		mciSendCommand(op.wDeviceID,MCI_CLOSE,MCI_WAIT,0);
	}

	if (m_pMediaStatus)
		*m_pMediaStatus = IsCDPresent(m_chDriveName);

	return 0;
}
//-------------------------------------------------------------------
bool IsCDPresent(TCHAR chDriveName)
{
	bool bResult = false;
	MCI_OPEN_PARMS op;
	MCI_STATUS_PARMS param;
	DWORD flags;
	TCHAR szDriveName[4];

	::memset(szDriveName , 0 , sizeof(szDriveName));
	::memset(&op, 0, sizeof(op));
	::memset(&param, 0, sizeof(param));

	op.lpstrDeviceType = "cdaudio";
	
	::lstrcpy(szDriveName , "?:");
	szDriveName[0] = chDriveName;

	op.lpstrElementName = szDriveName;
	flags = MCI_OPEN_TYPE  | MCI_OPEN_ELEMENT | MCI_OPEN_SHAREABLE;

	mciSendCommand(0,MCI_OPEN,flags,(unsigned long)&op);

	if (op.wDeviceID)
	{
		param.dwItem = MCI_STATUS_MEDIA_PRESENT;
		MCIERROR err = mciSendCommand(op.wDeviceID, MCI_STATUS, MCI_STATUS_ITEM , (DWORD) (LPMCI_STATUS_PARMS) &param);
		if ((err == 0) && (param.dwReturn == TRUE))
			bResult = true;

		mciSendCommand(op.wDeviceID, MCI_CLOSE, MCI_WAIT, 0);
	}

	return bResult;
};
//-------------------------------------------------------------------

//------ CCDThread class --------------------------------------------
CCDThread::CCDThread() :
	m_chDriveName  ('?'),
	m_bOpen        (false),
	m_pMediaStatus (0),
	CAxThread      ()
{
}
//-------------------------------------------------------------------
CCDThread::~CCDThread()
{
}
//-------------------------------------------------------------------
bool CCDThread::Create(TCHAR chDriveName, bool bOpen, bool* pMediaStatus)
{
	m_chDriveName  = chDriveName;
	m_bOpen        = bOpen;
	m_pMediaStatus = pMediaStatus;
	return CAxThread::Create();
}
//-------------------------------------------------------------------

//--- CAxCDManager class --------------------------------------------
CAxCDManager::CAxCDManager()
{
	RefreshState();
}
//-------------------------------------------------------------------
CAxCDManager::CAxCDManager(bool bUseScrlLock) 
{
	RefreshState();
}
//-------------------------------------------------------------------
CAxCDManager::~CAxCDManager()
{
}
//-------------------------------------------------------------------
bool CAxCDManager::OpenCloseCd(int iDrive, bool bOpen, bool* pMediaStatus)
{
	if (
		(iDrive > -1) && 
		(iDrive < m_iDrivesCount)
		)
	{
		if (
			(m_aCDThreads[iDrive].get()) &&
			(m_aCDThreads[iDrive]->IsRunning())
			)
			return false;

		m_aCDThreads[iDrive] = TThreadPtr(new CCDThread());
		return m_aCDThreads[iDrive]->Create(m_aCDLetters[iDrive], bOpen, pMediaStatus);
	}

	return false;
}
//-------------------------------------------------------------------
bool CAxCDManager::IsRunning() const
{
	if (m_iDrivesCount > 0)
	{
		for (int i = 0; i < m_iDrivesCount; i++)
		if (
			(m_aCDThreads[i].get()) &&
			(m_aCDThreads[i]->IsRunning())
			)
			return true;
	}

	return false;
}
//-------------------------------------------------------------------
int CAxCDManager::GetCDLetters(char *pLetters, int iSize) const
{
	if (iSize >= MAX_DRIVES)
	{
		int iPos = 0;
		::memset(pLetters, 0, iSize);
		DWORD dwDrives = ::GetLogicalDrives();
		for (int i = 0; i < MAX_DRIVES; i++)
		{
			if (dwDrives & (1 << i))
			{
				TCHAR chDrive = 65 + i;
				TCHAR szPath[4];
				::lstrcpy(szPath, "?:\\");
				szPath[0] = chDrive;

				if (::GetDriveType(szPath) == DRIVE_CDROM)
				{
					*(pLetters + iPos) = chDrive;
					iPos++;
				}
			}
		}
		return iPos;
	}

	return 0;
}
//-------------------------------------------------------------------
bool CAxCDManager::SelectDrive(int iDrive)
{
	if (
		(iDrive > -1) && 
		(iDrive < m_iDrivesCount)
		)
	{
		m_iCurrDrive = iDrive;
		return true;
	}

	return false;
}
//-------------------------------------------------------------------
bool CAxCDManager::SelectDrive(TCHAR chDrive)
{
	if (m_iDrivesCount)
	{
		::CharUpper(&chDrive);
		for (int i = 0; i < m_iDrivesCount; i++)
			if (m_aCDLetters[i] == chDrive)
			{
				SelectDrive(i);
				return true;
			}
	}

	return false;
}
//-------------------------------------------------------------------
bool CAxCDManager::IsOpened() const
{
	if (m_iCurrDrive > -1)
	{
		return m_aOpenStatus[m_iCurrDrive];
	}

	return false;
}
//-------------------------------------------------------------------
bool CAxCDManager::IsCDPresent() const
{
	if (m_iCurrDrive > -1)
	{
		return m_aMediaStatus[m_iCurrDrive];
	}

	return false;
}
//-------------------------------------------------------------------
bool CAxCDManager::Open()
{
	if (m_iCurrDrive > -1)
	{
		if (OpenCloseCd(m_iCurrDrive, true))
		{
			m_aOpenStatus[m_iCurrDrive]  = true;
			m_aMediaStatus[m_iCurrDrive] = false;
			return true;
		}
	}

	return false;
}
//-------------------------------------------------------------------
bool CAxCDManager::Close(bool bDetectCD)
{
	if (m_iCurrDrive > -1)
	{
		if (OpenCloseCd(m_iCurrDrive, false, 
			bDetectCD ? &m_aMediaStatus[m_iCurrDrive] : 0))
		{
			m_aOpenStatus[m_iCurrDrive] = false;
			return true;
		}
	}

	return false;
}
//-------------------------------------------------------------------
bool CAxCDManager::OpenClose(bool bDetectCD)
{
	if (m_iCurrDrive > -1)
	{
		bool bResult = false;
		if (m_aOpenStatus[m_iCurrDrive])
			bResult = OpenCloseCd(m_iCurrDrive, false,
				bDetectCD ? &m_aMediaStatus[m_iCurrDrive] : 0);
		else
			bResult = OpenCloseCd(m_iCurrDrive, true,
				bDetectCD ? &m_aMediaStatus[m_iCurrDrive] : 0);

		if (bResult)
		{
			m_aOpenStatus[m_iCurrDrive] = !m_aOpenStatus[m_iCurrDrive];
			return true;
		}
	}

	return false;
}
//-------------------------------------------------------------------
void CAxCDManager::RefreshState()
{
	m_iCurrDrive   = -1;
	::memset(m_aOpenStatus, false, sizeof(m_aOpenStatus));
	::memset(m_aMediaStatus, false, sizeof(m_aMediaStatus));
	m_iDrivesCount = GetCDLetters(m_aCDLetters, sizeof(m_aCDLetters));
}
//-------------------------------------------------------------------
int CAxCDManager::GetDrivesCount() const
{
	return m_iDrivesCount;
}
//-------------------------------------------------------------------
TCHAR CAxCDManager::GetDriveName(int iDrive) const
{
	if (
		(iDrive > -1) && 
		(iDrive < m_iDrivesCount)
		)
		return m_aCDLetters[iDrive];

	return '?';
}
//-------------------------------------------------------------------
TCHAR CAxCDManager::operator[] (int i) const
{
	return GetDriveName(i);
}
//-------------------------------------------------------------------
TCHAR CAxCDManager::GetSelectedDrive() const
{
	if (m_iCurrDrive > -1)
	{
		return m_aCDLetters[m_iCurrDrive];
	}

	return '?';
}
//-------------------------------------------------------------------
}