// AxThread.cpp: implementation of the CAxThread class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <AxThread.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//--------------------------------------------------------
namespace AxWinLib {

DWORD WINAPI ThreadProc(LPVOID pParam)
{
	CAxThread* pThread = 0;
	pThread = (CAxThread*)pParam;
	DWORD dwResult = CAxThread::UNKNOWN;

	if (pThread)
		dwResult = pThread->Execute();

	return dwResult;
}
//--------------------------------------------------------
bool CAxThread::Create(bool bSuspended)
{
	if (m_hHandle == NULL)
	{
		m_hHandle = ::CreateThread(0, 0, ThreadProc, this,
			bSuspended ? CREATE_SUSPENDED : 0, &m_dwID);
		return (m_hHandle != NULL);
	}
	
	return false;
}
//--------------------------------------------------------
void CAxThread::Resume()
{
	if (m_hHandle)
		::ResumeThread(m_hHandle);
}
//--------------------------------------------------------
void CAxThread::Suspend()
{
	if (m_hHandle)
		::SuspendThread(m_hHandle);
}
//--------------------------------------------------------
void CAxThread::Terminate()
{
	if (m_hHandle)
		::TerminateThread(m_hHandle, TERMINATED);
}
//--------------------------------------------------------
void CAxThread::SoftTerminate()
{
	m_bDoExit = true;
}
//--------------------------------------------------------
CAxThread::CAxThread() : 
	m_hHandle    (NULL),
	m_dwID       (0),
	m_bDoExit    (false)
{
}
//--------------------------------------------------------
CAxThread::~CAxThread()
{
	ResetMembers();
}
//--------------------------------------------------------
HANDLE CAxThread::GetHandle() const
{
	return m_hHandle;
}
//--------------------------------------------------------
DWORD CAxThread::GetID() const
{
	return m_dwID;
}
//--------------------------------------------------------
void CAxThread::ResetMembers()
{
	if (m_hHandle != NULL)
	{
		::TerminateThread(m_hHandle, TERMINATED);
		::CloseHandle(m_hHandle);
	}

	m_hHandle = NULL;
	m_bDoExit = false;
	m_dwID    = 0;
}
//--------------------------------------------------------
bool CAxThread::IsRunning() const
{
	DWORD dwExitCode = UNKNOWN;
	::GetExitCodeThread(m_hHandle, &dwExitCode);

	return dwExitCode == STILL_ACTIVE;
}
//--------------------------------------------------------
bool CAxThread::IsRequestExit() const
{
	return m_bDoExit;
}
//--------------------------------------------------------
CAxThread::operator HANDLE()
{
	return m_hHandle;
}
//--------------------------------------------------------
bool CAxThread::operator==(const CAxThread& refThread) const
{
	return (m_hHandle == refThread.m_hHandle);
}

}